Files
project_6/cccl_upstream/cudax/examples/stf/logical_gates_composition.cu
muh-bot dedf08166a [CCCL] Add missing CCCL components: c2h, nvbench_helper, cmake, cudax, AGENTS.md
Added 863 files from NVIDIA/cccl sparse checkout:
- c2h/ (27 files): Catch2 test helpers — generators, validators, runner
- nvbench_helper/ (10 files): Benchmark harness utilities
- cmake/ (29 files): CMake presets and build helpers
- cudax/ (794 files): Experimental CUDA extensions
- AGENTS.md: NVIDIA's official AI agent instructions for CCCL
- CMakePresets.json: Standardized build configurations
- cccl-version.json: Version tracking

Also added CCCL_ASSET_MAP.md mapping all 4295 CCCL files to
competition value and PRD items.

cccl_upstream now covers 100% of competition-critical assets:
- 27 tuning headers (SM80/90/100 benchmark data)
- 32 dispatch headers (algorithm implementations)
- 60 Thrust examples (correctness verification)
- 217 CUB Catch2 tests (regression matrix)
- 153 CUB benchmarks (parameter space search)
- 18 CUB examples (API verification)
- 27 test helpers + benchmark harness
- 794 cudax experimental extensions
2026-08-06 02:14:18 +00:00

74 lines
1.8 KiB
Plaintext

//===----------------------------------------------------------------------===//
//
// Part of CUDASTF in CUDA C++ Core Libraries,
// under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
// SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES.
//
//===----------------------------------------------------------------------===//
/**
* @file
* @brief Composition of boolean operations applied on logical data
*/
#include <cuda/experimental/stf.cuh>
using namespace cuda::experimental::stf;
// z = AND(x,y)
logical_data<slice<int>> AND(context& ctx, logical_data<slice<int>> x, logical_data<slice<int>> y)
{
assert(x.shape().size() == y.shape().size());
auto z = ctx.logical_data(x.shape());
std::string symbol = "(" + x.get_symbol() + " & " + y.get_symbol() + ")";
z.set_symbol(symbol);
ctx.parallel_for(z.shape(), x.read(), y.read(), z.write()).set_symbol("AND")->*
[] __device__(size_t i, auto dx, auto dy, auto dz) {
dz(i) = dx(i) & dy(i);
};
return z;
}
// y = NOT(x)
logical_data<slice<int>> NOT(context& ctx, logical_data<slice<int>> x)
{
auto y = ctx.logical_data(x.shape());
std::string symbol = "( !" + x.get_symbol() + ")";
y.set_symbol(symbol);
ctx.parallel_for(y.shape(), x.read(), y.write()).set_symbol("NOT")->*[] __device__(size_t i, auto dx, auto dy) {
dy(i) = ~dx(i);
};
return y;
}
int main()
{
const size_t n = 12;
int X[n], Y[n], Z[n];
context ctx;
auto lX = ctx.logical_data(X);
auto lY = ctx.logical_data(Y);
auto lZ = ctx.logical_data(Z);
lX.set_symbol("X");
lY.set_symbol("Y");
lZ.set_symbol("Z");
auto lB = AND(ctx, AND(ctx, lX, lY), AND(ctx, lX, lZ));
auto lC = AND(ctx, NOT(ctx, AND(ctx, lB, NOT(ctx, lY))), lX);
ctx.finalize();
}